@@ -1,35 +1,40 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * chicken.cxx
  *
  * CHICKEN language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 10270 2008-02-27 15:29:55Z wsfulton $";
+char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
 
 #include "swigmod.h"
 
 #include <ctype.h>
 
-static const char *chicken_usage = (char *) "\
+static const char *usage = (char *) "\
 \
 CHICKEN Options (available with -chicken)\n\
-     -proxy                 - Export TinyCLOS class definitions\n\
      -closprefix <prefix>   - Prepend <prefix> to all clos identifiers\n\
-     -useclassprefix        - Prepend the class name to all clos identifiers\n\
-     -unhideprimitive       - Unhide the primitive: symbols\n\
-     -nounit                - Do not (declare (unit ...)) in scheme file\n\
      -noclosuses            - Do not (declare (uses ...)) in scheme file\n\
      -nocollection          - Do not register pointers with chicken garbage\n\
                               collector and export destructors\n\
+     -nounit                - Do not (declare (unit ...)) in scheme file\n\
+     -proxy                 - Export TinyCLOS class definitions\n\
+     -unhideprimitive       - Unhide the primitive: symbols\n\
+     -useclassprefix        - Prepend the class name to all clos identifiers\n\
 \n";
 
 static char *module = 0;
 static char *chicken_path = (char *) "chicken";
 static int num_methods = 0;
 
+static File *f_begin = 0;
 static File *f_runtime = 0;
 static File *f_header = 0;
 static File *f_wrappers = 0;
@@ -101,12 +106,12 @@ protected:
   int isPointer(SwigType *t);
   void dispatchFunction(Node *n);
 
-  String *chickenNameMapping(String *, String_or_char *);
+  String *chickenNameMapping(String *, const_String_or_char_ptr );
   String *chickenPrimitiveName(String *);
 
   String *runtimeCode();
   String *defaultExternalRuntimeFilename();
-  String *buildClosFunctionCall(List *types, String_or_char *closname, String_or_char *funcname);
+  String *buildClosFunctionCall(List *types, const_String_or_char_ptr closname, const_String_or_char_ptr funcname);
 };
 
 /* -----------------------------------------------------------------------
@@ -132,7 +137,7 @@ void CHICKEN::main(int argc, char *argv[]) {
   for (i = 1; i < argc; i++) {
     if (argv[i]) {
       if (strcmp(argv[i], "-help") == 0) {
-	fputs(chicken_usage, stdout);
+	fputs(usage, stdout);
 	SWIG_exit(0);
       } else if (strcmp(argv[i], "-proxy") == 0) {
 	clos = 1;
@@ -188,11 +193,12 @@ int CHICKEN::top(Node *n) {
   /* Initialize all of the output files */
   String *outfile = Getattr(n, "outfile");
 
-  f_runtime = NewFile(outfile, "w");
-  if (!f_runtime) {
+  f_begin = NewFile(outfile, "w", SWIG_output_files());
+  if (!f_begin) {
     FileErrorDisplay(outfile);
     SWIG_exit(EXIT_FAILURE);
   }
+  f_runtime = NewString("");
   f_init = NewString("");
   f_header = NewString("");
   f_wrappers = NewString("");
@@ -205,6 +211,7 @@ int CHICKEN::top(Node *n) {
   /* Register file targets with the SWIG file handler */
   Swig_register_filebyname("header", f_header);
   Swig_register_filebyname("wrapper", f_wrappers);
+  Swig_register_filebyname("begin", f_begin);
   Swig_register_filebyname("runtime", f_runtime);
   Swig_register_filebyname("init", f_init);
 
@@ -215,14 +222,16 @@ int CHICKEN::top(Node *n) {
   clos_methods = NewString("");
   scm_const_defs = NewString("");
 
-  Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
-  Swig_banner(f_runtime);
+  Swig_banner(f_begin);
 
-  Printf(f_runtime, "/* Implementation : CHICKEN */\n\n");
+  Printf(f_runtime, "\n");
+  Printf(f_runtime, "#define SWIGCHICKEN\n");
 
   if (no_collection)
     Printf(f_runtime, "#define SWIG_CHICKEN_NO_COLLECTION 1\n");
 
+  Printf(f_runtime, "\n");
+
   /* Set module name */
   module = Swig_copy_string(Char(Getattr(n, "name")));
   scmmodule = NewString(module);
@@ -251,14 +260,14 @@ int CHICKEN::top(Node *n) {
   Printf(f_init, "#endif\n");
 
   Printf(chicken_filename, "%s%s.scm", SWIG_output_directory(), module);
-  if ((f_scm = NewFile(chicken_filename, "w")) == 0) {
+  if ((f_scm = NewFile(chicken_filename, "w", SWIG_output_files())) == 0) {
     FileErrorDisplay(chicken_filename);
     SWIG_exit(EXIT_FAILURE);
   }
 
-  Printv(f_scm,
-	 ";; -*- buffer-read-only: t -*- vi: set ro:\n",
-	 ";; This file was created automatically by SWIG.\n", ";; Don't modify this file, modify the SWIG interface instead.\n", NIL);
+  Swig_banner_target_lang(f_scm, ";;");
+  Printf(f_scm, "\n");
+
   if (declare_unit)
     Printv(f_scm, "(declare (unit ", scmmodule, "))\n\n", NIL);
   Printv(f_scm, "(declare \n",
@@ -307,15 +316,17 @@ int CHICKEN::top(Node *n) {
   /* Close all of the files */
   Delete(primitive_names);
   Delete(scmmodule);
-  Dump(f_header, f_runtime);
-  Dump(f_wrappers, f_runtime);
-  Wrapper_pretty_print(f_init, f_runtime);
+  Dump(f_runtime, f_begin);
+  Dump(f_header, f_begin);
+  Dump(f_wrappers, f_begin);
+  Wrapper_pretty_print(f_init, f_begin);
   Delete(f_header);
   Delete(f_wrappers);
   Delete(f_sym_size);
   Delete(f_init);
-  Close(f_runtime);
+  Close(f_begin);
   Delete(f_runtime);
+  Delete(f_begin);
   return SWIG_OK;
 }
 
@@ -329,7 +340,6 @@ int CHICKEN::functionWrapper(Node *n) {
   Parm *p;
   int i;
   String *wname;
-  String *source;
   Wrapper *f;
   String *mangle = NewString("");
   String *get_pointers;
@@ -368,7 +378,7 @@ int CHICKEN::functionWrapper(Node *n) {
   Wrapper_add_local(f, "resultobj", "C_word resultobj");
 
   /* Write code to extract function parameters. */
-  emit_args(d, l, f);
+  emit_parameter_variables(l, f);
 
   /* Attach the standard typemaps */
   emit_attach_parmmaps(l, f);
@@ -398,8 +408,6 @@ int CHICKEN::functionWrapper(Node *n) {
     SwigType *pt = Getattr(p, "type");
     String *ln = Getattr(p, "lname");
 
-    source = NewStringf("scm%d", i + 1);
-
     Printf(f->def, ", C_word scm%d", i + 1);
     Printf(declfunc, ",C_word");
 
@@ -407,6 +415,7 @@ int CHICKEN::functionWrapper(Node *n) {
     if ((tm = Getattr(p, "tmap:in"))) {
       String *parse = Getattr(p, "tmap:in:parse");
       if (!parse) {
+        String *source = NewStringf("scm%d", i + 1);
 	Replaceall(tm, "$source", source);
 	Replaceall(tm, "$target", ln);
 	Replaceall(tm, "$input", source);
@@ -445,20 +454,15 @@ int CHICKEN::functionWrapper(Node *n) {
 	    }
 	  }
 	}
-
-      } else {
+        Delete(source);
       }
 
-
       p = Getattr(p, "tmap:in:next");
       continue;
     } else {
       Swig_warning(WARN_TYPEMAP_IN_UNDEF, input_file, line_number, "Unable to use type %s as a function argument.\n", SwigType_str(pt, 0));
       break;
     }
-
-    Delete(source);
-    p = nextSibling(p);
   }
 
   /* finish argument marshalling */
@@ -501,11 +505,6 @@ int CHICKEN::functionWrapper(Node *n) {
     }
   }
 
-  Setattr(n, "wrap:name", wname);
-
-  /* Emit the function call */
-  emit_action(n, f);
-
   /* Insert argument output code */
   have_argout = 0;
   for (p = l; p;) {
@@ -514,7 +513,7 @@ int CHICKEN::functionWrapper(Node *n) {
       if (!have_argout) {
 	have_argout = 1;
 	// Print initial argument output code
-	Printf(f->code, "SWIG_Chicken_SetupArgout\n");
+	Printf(argout, "SWIG_Chicken_SetupArgout\n");
       }
 
       Replaceall(tm, "$source", Getattr(p, "lname"));
@@ -528,9 +527,14 @@ int CHICKEN::functionWrapper(Node *n) {
     }
   }
 
+  Setattr(n, "wrap:name", wname);
+
+  /* Emit the function call */
+  String *actioncode = emit_action(n);
+
   /* Return the function value */
-  if ((tm = Swig_typemap_lookup_new("out", n, "result", 0))) {
-    Replaceall(tm, "$source", "result");
+  if ((tm = Swig_typemap_lookup_out("out", n, Swig_cresult_name(), f, actioncode))) {
+    Replaceall(tm, "$source", Swig_cresult_name());
     Replaceall(tm, "$target", "resultobj");
     Replaceall(tm, "$result", "resultobj");
     if (GetFlag(n, "feature:new")) {
@@ -547,6 +551,7 @@ int CHICKEN::functionWrapper(Node *n) {
   } else {
     Swig_warning(WARN_TYPEMAP_OUT_UNDEF, input_file, line_number, "Unable to use return type %s in function %s.\n", SwigType_str(d, 0), name);
   }
+  emit_return_variable(n, d, f);
 
   /* Insert the argumetn output code */
   Printv(f->code, argout, NIL);
@@ -556,15 +561,15 @@ int CHICKEN::functionWrapper(Node *n) {
 
   /* Look to see if there is any newfree cleanup code */
   if (GetFlag(n, "feature:new")) {
-    if ((tm = Swig_typemap_lookup_new("newfree", n, "result", 0))) {
-      Replaceall(tm, "$source", "result");
+    if ((tm = Swig_typemap_lookup("newfree", n, Swig_cresult_name(), 0))) {
+      Replaceall(tm, "$source", Swig_cresult_name());
       Printf(f->code, "%s\n", tm);
     }
   }
 
   /* See if there is any return cleanup code */
-  if ((tm = Swig_typemap_lookup_new("ret", n, "result", 0))) {
-    Replaceall(tm, "$source", "result");
+  if ((tm = Swig_typemap_lookup("ret", n, Swig_cresult_name(), 0))) {
+    Replaceall(tm, "$source", Swig_cresult_name());
     Printf(f->code, "%s\n", tm);
   }
 
@@ -672,16 +677,13 @@ int CHICKEN::variableWrapper(Node *n) {
   String *wname = NewString("");
   String *mangle = NewString("");
   String *tm;
-  String *tm2 = NewString("");;
+  String *tm2 = NewString("");
   String *argnum = NewString("0");
   String *arg = NewString("argv[0]");
   Wrapper *f;
   String *overname = 0;
   String *scmname;
 
-  int num_required;
-  int num_arguments;
-
   scmname = NewString(iname);
   Replaceall(scmname, "_", "-");
 
@@ -700,10 +702,6 @@ int CHICKEN::variableWrapper(Node *n) {
   emit_attach_parmmaps(l, f);
   Setattr(n, "wrap:parms", l);
 
-  /* Get number of required and total arguments */
-  num_arguments = emit_num_arguments(l);
-  num_required = emit_num_required(l);
-
   // evaluation function names
   Append(wname, Swig_name_wrapper(iname));
   if (overname) {
@@ -726,12 +724,12 @@ int CHICKEN::variableWrapper(Node *n) {
     /* Check for a setting of the variable value */
     if (!GetFlag(n, "feature:immutable")) {
       Printf(f->code, "if (argc > 2) {\n");
-      if ((tm = Swig_typemap_lookup_new("varin", n, name, 0))) {
+      if ((tm = Swig_typemap_lookup("varin", n, name, 0))) {
 	Replaceall(tm, "$source", "value");
 	Replaceall(tm, "$target", name);
 	Replaceall(tm, "$input", "value");
 	/* Printv(f->code, tm, "\n",NIL); */
-	emit_action_code(n, f, tm);
+	emit_action_code(n, f->code, tm);
       } else {
 	Swig_warning(WARN_TYPEMAP_VARIN_UNDEF, input_file, line_number, "Unable to set variable of type %s.\n", SwigType_str(t, 0));
       }
@@ -747,13 +745,13 @@ int CHICKEN::variableWrapper(Node *n) {
 
     // Now return the value of the variable - regardless
     // of evaluating or setting.
-    if ((tm = Swig_typemap_lookup_new("varout", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup("varout", n, name, 0))) {
       Replaceall(tm, "$source", varname);
       Replaceall(tm, "$varname", varname);
       Replaceall(tm, "$target", "resultobj");
       Replaceall(tm, "$result", "resultobj");
       /* Printf(f->code, "%s\n", tm); */
-      emit_action_code(n, f, tm);
+      emit_action_code(n, f->code, tm);
     } else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number, "Unable to read variable of type %s\n", SwigType_str(t, 0));
     }
@@ -840,9 +838,6 @@ int CHICKEN::constantWrapper(Node *n) {
   String *rvalue;
   SwigType *nctype;
 
-  int num_required;
-  int num_arguments;
-
   scmname = NewString(iname);
   Replaceall(scmname, "_", "-");
 
@@ -868,9 +863,10 @@ int CHICKEN::constantWrapper(Node *n) {
     Delete(SwigType_pop(nctype));
   }
 
+  bool is_enum_item = (Cmp(nodeType(n), "enumitem") == 0);
   if (SwigType_type(nctype) == T_STRING) {
     rvalue = NewStringf("\"%s\"", value);
-  } else if (SwigType_type(nctype) == T_CHAR) {
+  } else if (SwigType_type(nctype) == T_CHAR && !is_enum_item) {
     rvalue = NewStringf("\'%s\'", value);
   } else {
     rvalue = NewString(value);
@@ -880,7 +876,7 @@ int CHICKEN::constantWrapper(Node *n) {
   if (SwigType_type(t) == T_MPOINTER) {
     Printf(f_header, "static %s = %s;\n", SwigType_str(t, source), rvalue);
   } else {
-    if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup("constcode", n, name, 0))) {
       Replaceall(tm, "$source", rvalue);
       Replaceall(tm, "$target", source);
       Replaceall(tm, "$result", source);
@@ -898,10 +894,6 @@ int CHICKEN::constantWrapper(Node *n) {
   emit_attach_parmmaps(l, f);
   Setattr(n, "wrap:parms", l);
 
-  /* Get number of required and total arguments */
-  num_arguments = emit_num_arguments(l);
-  num_required = emit_num_required(l);
-
   // evaluation function names
 
   // Check for interrupts
@@ -919,14 +911,14 @@ int CHICKEN::constantWrapper(Node *n) {
     Printf(f->code, "if (argc!=2) C_bad_argc(argc,2);\n");
 
     // Return the value of the variable
-    if ((tm = Swig_typemap_lookup_new("varout", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup("varout", n, name, 0))) {
 
       Replaceall(tm, "$source", source);
       Replaceall(tm, "$varname", source);
       Replaceall(tm, "$target", "resultobj");
       Replaceall(tm, "$result", "resultobj");
       /* Printf(f->code, "%s\n", tm); */
-      emit_action_code(n, f, tm);
+      emit_action_code(n, f->code, tm);
     } else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number, "Unable to read variable of type %s\n", SwigType_str(t, 0));
     }
@@ -1147,9 +1139,9 @@ int CHICKEN::membervariableHandler(Node *n) {
 
   //String *getfunc = NewStringf("%s-%s-get", short_class_name, proc);
   //String *setfunc = NewStringf("%s-%s-set", short_class_name, proc);
-  String *getfunc = Swig_name_get(Swig_name_member(c_class_name, iname));
+  String *getfunc = Swig_name_get(NSPACE_TODO, Swig_name_member(NSPACE_TODO, c_class_name, iname));
   Replaceall(getfunc, "_", "-");
-  String *setfunc = Swig_name_set(Swig_name_member(c_class_name, iname));
+  String *setfunc = Swig_name_set(NSPACE_TODO, Swig_name_member(NSPACE_TODO, c_class_name, iname));
   Replaceall(setfunc, "_", "-");
 
   Printv(clos_class_defines, "        (list '", proc, " ':swig-virtual ':swig-get ", chickenPrimitiveName(getfunc), NIL);
@@ -1196,7 +1188,7 @@ int CHICKEN::constructorHandler(Node *n) {
   has_constructor_args = 1;
 
   String *iname = Getattr(n, "sym:name");
-  constructor_name = Swig_name_construct(iname);
+  constructor_name = Swig_name_construct(NSPACE_TODO, iname);
   Replaceall(constructor_name, "_", "-");
   return SWIG_OK;
 }
@@ -1239,7 +1231,7 @@ int CHICKEN::importDirective(Node *n) {
   return Language::importDirective(n);
 }
 
-String *CHICKEN::buildClosFunctionCall(List *types, String_or_char *closname, String_or_char *funcname) {
+String *CHICKEN::buildClosFunctionCall(List *types, const_String_or_char_ptr closname, const_String_or_char_ptr funcname) {
   String *method_signature = NewString("");
   String *func_args = NewString("");
   String *func_call = NewString("");
@@ -1377,12 +1369,10 @@ void CHICKEN::dispatchFunction(Node *n) {
       SortList(flist, compareTypeLists);
 
       String *clos_name;
-      int construct = 0;
       if (have_constructor && !has_constructor_args) {
 	has_constructor_args = 1;
 	constructor_dispatch = NewStringf("%s@SWIG@new@dispatch", short_class_name);
 	clos_name = Copy(constructor_dispatch);
-	construct = 1;
 	Printf(clos_methods, "(declare (hide %s))\n", clos_name);
       } else if (in_class)
 	clos_name = NewString(member_name);
@@ -1509,11 +1499,11 @@ int CHICKEN::validIdentifier(String *s) {
   /* ------------------------------------------------------------
    * closNameMapping()
    * Maps the identifier from C++ to the CLOS based on command 
-   * line paramaters and such.
+   * line parameters and such.
    * If class_name = "" that means the mapping is for a function or
    * variable not attached to any class.
    * ------------------------------------------------------------ */
-String *CHICKEN::chickenNameMapping(String *name, String_or_char *class_name) {
+String *CHICKEN::chickenNameMapping(String *name, const_String_or_char_ptr class_name) {
   String *n = NewString("");
 
   if (Strcmp(class_name, "") == 0) {
