@@ -11,7 +11,7 @@
  * Useful functions for emitting various pieces of code.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_emit_cxx[] = "$Id: emit.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
+char cvsroot_emit_cxx[] = "$Id: emit.cxx 13511 2012-08-05 08:37:59Z oliv3rb $";
 
 #include "swigmod.h"
 
@@ -470,6 +470,7 @@ String *emit_action(Node *n) {
 
   if (catchlist) {
     int unknown_catch = 0;
+    int has_varargs = 0;
     Printf(eaction, "}\n");
     for (Parm *ep = catchlist; ep; ep = nextSibling(ep)) {
       String *em = Swig_typemap_lookup("throws", ep, "_e", 0);
@@ -480,6 +481,7 @@ String *emit_action(Node *n) {
           Printf(eaction, "catch(%s) {", SwigType_str(et, "_e"));
         } else if (SwigType_isvarargs(etr)) {
           Printf(eaction, "catch(...) {");
+          has_varargs = 1;
         } else {
           Printf(eaction, "catch(%s) {", SwigType_str(et, "&_e"));
         }
@@ -490,8 +492,8 @@ String *emit_action(Node *n) {
         unknown_catch = 1;
       }
     }
-    if (unknown_catch) {
-    Printf(eaction, "catch(...) { throw; }\n");
+    if (unknown_catch && !has_varargs) {
+      Printf(eaction, "catch(...) { throw; }\n");
     }
   }
 
