@@ -11,6 +11,8 @@
  * Mzscheme language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_mzscheme_cxx[] = "$Id: mzscheme.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
+
 #include "swigmod.h"
 
 #include <ctype.h>
@@ -31,10 +33,12 @@ static String *convert_proto_tab = 0;
 static String *struct_name = 0;
 static String *mangled_struct_name = 0;
 
-static String *prefix = 0;
+static char *prefix = 0;
 static bool declaremodule = false;
 static bool noinit = false;
-static String *load_libraries = NULL;
+//DLOPEN PATCH
+static char *load_libraries = NULL;
+//DLOPEN PATCH
 static String *module = 0;
 static char *mzscheme_path = (char *) "mzscheme";
 static String *init_func_def = 0;
@@ -71,7 +75,8 @@ public:
 	  SWIG_exit(0);
 	} else if (strcmp(argv[i], "-prefix") == 0) {
 	  if (argv[i + 1]) {
-	    prefix = NewString(argv[i + 1]);
+	    prefix = new char[strlen(argv[i + 1]) + 2];
+	    strcpy(prefix, argv[i + 1]);
 	    Swig_mark_arg(i);
 	    Swig_mark_arg(i + 1);
 	    i++;
@@ -85,26 +90,26 @@ public:
 	  noinit = true;
 	  Swig_mark_arg(i);
 	}
+// DLOPEN PATCH
 	else if (strcmp(argv[i], "-dynamic-load") == 0) {
-	  if (argv[i + 1]) {
-	    Delete(load_libraries);
-	    load_libraries = NewString(argv[i + 1]);
-	    Swig_mark_arg(i++);
-	    Swig_mark_arg(i);
-	  } else {
-	    Swig_arg_error();
-	  }
+	  load_libraries = new char[strlen(argv[i + 1]) + 2];
+	  strcpy(load_libraries, argv[i + 1]);
+	  Swig_mark_arg(i++);
+	  Swig_mark_arg(i);
 	}
+// DLOPEN PATCH
       }
     }
 
-    // If a prefix has been specified make sure it ends in a '_' (not actually used!)
+    // If a prefix has been specified make sure it ends in a '_'
+
     if (prefix) {
-      const char *px = Char(prefix);
-      if (px[Len(prefix) - 1] != '_')
-	Printf(prefix, "_");
+      if (prefix[strlen(prefix)] != '_') {
+	prefix[strlen(prefix) + 1] = 0;
+	prefix[strlen(prefix)] = '_';
+      }
     } else
-      prefix = NewString("swig_");
+      prefix = (char *) "swig_";
 
     // Add a symbol for this module
 
@@ -172,9 +177,11 @@ public:
       Printf(f_init, "\treturn scheme_void;\n}\n");
       Printf(f_init, "Scheme_Object *scheme_initialize(Scheme_Env *env) {\n");
 
+      // DLOPEN PATCH
       if (load_libraries) {
 	Printf(f_init, "mz_set_dlopen_libraries(\"%s\");\n", load_libraries);
       }
+      // DLOPEN PATCH
 
       Printf(f_init, "\treturn scheme_reload(env);\n");
       Printf(f_init, "}\n");
@@ -196,6 +203,7 @@ public:
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
+    Close(f_begin);
     Delete(f_runtime);
     Delete(f_begin);
     return SWIG_OK;
@@ -237,12 +245,14 @@ public:
     int numreq;
     String *overname = 0;
 
+    // PATCH DLOPEN
     if (load_libraries) {
       ParmList *parms = Getattr(n, "parms");
       SwigType *type = Getattr(n, "type");
       String *name = NewString("caller");
       Setattr(n, "wrap:action", Swig_cresult(type, Swig_cresult_name(), Swig_cfunction_call(name, parms)));
     }
+    // PATCH DLOPEN
 
     // Make a wrapper name for this
     String *wname = Swig_name_wrapper(iname);
@@ -282,6 +292,7 @@ public:
     numargs = emit_num_arguments(l);
     numreq = emit_num_required(l);
 
+    // DLOPEN PATCH
     /* Add the holder for the pointer to the function to be opened */
     if (load_libraries) {
       Wrapper_add_local(f, "_function_loaded", "static int _function_loaded=(1==0)");
@@ -292,16 +303,19 @@ public:
 	Wrapper_add_local(f, "caller", SwigType_lstr(d, func));	/*"(*caller)()")); */
       }
     }
+    // DLOPEN PATCH
 
     // adds local variables
     Wrapper_add_local(f, "lenv", "int lenv = 1");
     Wrapper_add_local(f, "values", "Scheme_Object *values[MAXVALUES]");
 
+    // DLOPEN PATCH
     if (load_libraries) {
       Printf(f->code, "if (!_function_loaded) { _the_function=mz_load_function(\"%s\");_function_loaded=(1==1); }\n", iname);
       Printf(f->code, "if (!_the_function) { scheme_signal_error(\"Cannot load C function '%s'\"); }\n", iname);
       Printf(f->code, "caller=_the_function;\n");
     }
+    // DLOPEN PATCH
 
     // Now write code to extract the parameters (this is super ugly)
 
