@@ -11,6 +11,8 @@
  * R language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_r_cxx[] = "$Id: r.cxx 13713 2012-08-20 21:20:19Z wsfulton $";
+
 #include "swigmod.h"
 
 static const double DEFAULT_NUMBER = .0000123456712312312323;
@@ -570,7 +572,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   Parm *p = parms;
   for (i = 0; p; p = nextSibling(p), ++i) {
     String *arg = Getattr(p, "name");
-    String *lname;
+    String *lname = NewString("");
 
     if (!arg && Cmp(Getattr(p, "type"), "void")) {
       lname = NewStringf("s_arg%d", i+1);
@@ -680,7 +682,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
        XXX  Have to be a little more clever so that we can deal with struct A * - the * is getting lost.
        Is this still true? If so, will a SwigType_push() solve things?
     */
-    Parm *bbase = NewParmNode(rettype, n);
+    Parm *bbase = NewParm(rettype, Swig_cresult_name(), n);
     String *returnTM = Swig_typemap_lookup("in", bbase, Swig_cresult_name(), f);
     if(returnTM) {
       String *tm = returnTM;
@@ -724,7 +726,6 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   Delete(rtype);
   Delete(rettype);
   Delete(funcparams);
-  DelWrapper(f);
   
   return funName;
 }
@@ -836,6 +837,7 @@ int R::top(Node *n) {
   Delete(f_init);
 
   Delete(s_header);
+  Close(f_begin);
   Delete(f_runtime);
   Delete(f_begin);
 
@@ -869,7 +871,8 @@ int R::DumpCode(Node *n) {
   Printf(scode, "%s\n\n", s_classes);
   Printf(scode, "%s\n", sfile);
   
-  Delete(scode);
+  Close(scode);
+  //  Delete(scode);
   String *outfile = Getattr(n,"outfile");
   File *runtime = NewFile(outfile,"w", SWIG_output_files());
   if (!runtime) {
@@ -883,6 +886,7 @@ int R::DumpCode(Node *n) {
   Printf(runtime, "%s\n", f_wrapper);
   Printf(runtime, "%s\n", f_init);
 
+  Close(runtime);
   Delete(runtime);
 
   if(outputNamespaceInfo) {
@@ -903,6 +907,7 @@ int R::DumpCode(Node *n) {
     Printf(ns, "\nexportMethods(\n");
     writeListByLine(namespaceFunctions, ns, 1);
     Printf(ns, ")\n");
+    Close(ns);
     Delete(ns);
     Delete(s_namespace);
   }
@@ -1607,6 +1612,7 @@ void R::dispatchFunction(Node *n) {
 
 	String *tmcheck = Swig_typemap_lookup("rtypecheck", p, "", 0);
 	if (tmcheck) {
+
 	  String *tmp = NewString("");
 	  Printf(tmp, "argv[[%d]]", j+1);
 	  Replaceall(tmcheck, "$arg", tmp);
@@ -1623,34 +1629,32 @@ void R::dispatchFunction(Node *n) {
 		 tmcheck);
 	  p = Getattr(p, "tmap:in:next");
 	  continue;
-	}
-	if (tm) {
-	  if (Strcmp(tm,"numeric")==0) {
+	} 
+	if (DohStrcmp(tm,"numeric")==0) {
 	    Printf(f->code, "%sis.numeric(argv[[%d]])",
-		j == 0 ? "" : " && ",
-		j+1);
+		   j == 0 ? "" : " && ",
+		   j+1);
 	  }
-	  else if (Strcmp(tm,"integer")==0) {
+	  else if (DohStrcmp(tm,"integer")==0) {
 	    Printf(f->code, "%s(is.integer(argv[[%d]]) || is.numeric(argv[[%d]]))",
-		j == 0 ? "" : " && ",
-		j+1, j+1);
+		   j == 0 ? "" : " && ",
+		   j+1, j+1);
 	  }
-	  else if (Strcmp(tm,"character")==0) {
+	  else if (DohStrcmp(tm,"character")==0) {
 	    Printf(f->code, "%sis.character(argv[[%d]])",
-		j == 0 ? "" : " && ",
-		j+1);
+		   j == 0 ? "" : " && ",
+		   j+1);
 	  }
 	  else {
 	    Printf(f->code, "%sextends(argtypes[%d], '%s')",
-		j == 0 ? "" : " && ",
-		j+1,
-		tm);
+		   j == 0 ? "" : " && ",
+		   j+1,
+		   tm);
+	  }
+	  if (!SwigType_ispointer(Getattr(p, "type"))) {
+	    Printf(f->code, " && length(argv[[%d]]) == 1",
+		   j+1);
 	  }
-	}
-	if (!SwigType_ispointer(Getattr(p, "type"))) {
-	  Printf(f->code, " && length(argv[[%d]]) == 1",
-	      j+1);
-	}
 	p = Getattr(p, "tmap:in:next");
       }
       Printf(f->code, ") { f <- %s%s; }\n", sfname, overname);
@@ -1844,21 +1848,18 @@ int R::functionWrapper(Node *n) {
     String   *lname  = Getattr(p,"lname");
 
     // R keyword renaming
-    if (name) {
-      if (Swig_name_warning(p, 0, name, 0)) {
-	name = 0;
-      } else {
-	/* If we have a :: in the parameter name because we are accessing a static member of a class, say, then
-	   we need to remove that prefix. */
-	while (Strstr(name, "::")) {
-	  //XXX need to free.
-	  name = NewStringf("%s", Strchr(name, ':') + 2);
-	  if (debugMode)
-	    Printf(stdout, "+++  parameter name with :: in it %s\n", name);
-	}
-      }
+    if (name && Swig_name_warning(p, 0, name, 0))
+      name = 0;
+
+    /* If we have a :: in the parameter name because we are accessing a static member of a class, say, then
+       we need to remove that prefix. */
+    while (Strstr(name, "::")) {
+      //XXX need to free.
+      name = NewStringf("%s", Strchr(name, ':') + 2);
+      if (debugMode)
+	Printf(stdout, "+++  parameter name with :: in it %s\n", name);
     }
-    if (!name || Len(name) == 0)
+    if (Len(name) == 0)
       name = NewStringf("s_arg%d", i+1);
 
     name = replaceInitialDash(name);
@@ -1902,8 +1903,6 @@ int R::functionWrapper(Node *n) {
 	       "\n};\n",
 	       "if(is(", name, ", \"NativeSymbolInfo\")) {\n",
 	       name, " = ", name, "$address", ";\n}\n",
-	       "if(is(", name, ", \"ExternalReference\")) {\n",
-	       name, " = ", name, "@ref;\n}\n",
 	       "}; \n",
 	       NIL);
       } else {
@@ -2113,7 +2112,7 @@ int R::functionWrapper(Node *n) {
 	{ 
 	  String *finalizer = NewString(iname);
 	  Replace(finalizer, "new_", "", DOH_REPLACE_FIRST);
-	  Printf(sfun->code, "reg.finalizer(ans@ref, delete_%s)\n", finalizer);
+	  Printf(sfun->code, "reg.finalizer(ans, delete_%s)\n", finalizer);
 	}                                                                      
       Printf(sfun->code, "ans\n");
     }
@@ -2349,7 +2348,7 @@ int R::classDeclaration(Node *n) {
       elName = Getattr(c, "name");
  
       String *elKind = Getattr(c, "kind");
-      if (!Equal(elKind, "variable")) {
+      if (Strcmp(elKind, "variable") != 0) {
 	c = nextSibling(c);
 	continue;
       }
@@ -2451,7 +2450,7 @@ int R::generateCopyRoutines(Node *n) {
       continue;
     }
     String *elKind = Getattr(c, "kind");
-    if (!Equal(elKind, "variable")) {
+    if (Strcmp(elKind, "variable") != 0) {
       continue;
     }
 
